#!/usr/bin/env python
"""
_GetUnsubscribedDatasets_

Oracle implementation on GetUnsubscribedDatasets.
"""

from __future__ import division

from WMComponent.RucioInjector.Database.MySQL.GetUnsubscribedDatasets import GetUnsubscribedDatasets as MySQLBase


class GetUnsubscribedDatasets(MySQLBase):
    pass
