@CHARSET "UTF-8";

.masthead {
	background:rgb(229, 236, 249);
	border:solid rgb(100, 105, 115) 0px;
	border-bottom-width: 1px;
}
.masthead .mastheadLeft ul, .masthead .mastheadRight ul {
	border:solid rgb(129, 136, 149) 0px;
	color: rgb(100, 105, 115);
}
.masthead .mastheadLeft ul li a, .masthead .mastheadRight ul li a {
	color:rgb(100, 105, 115);
}
.masthead #dbs a{
	background:rgb(100, 105, 115);
	color:white;
}
/* This next bit is for site dependant masthead */
.mastheadfooter {
	background:rgb(100, 105, 115);
}
.mastheadfooter ul {
	color:white;
}
.mastheadfooter ul li {
	color:white;	
}
.mastheadfooter ul li a, .mastheadfooter ul li a:visited{
	color:white;
}
.mastheadfooter ul li a:hover{
	color:white;
}
.mastheadfooter ul li a:active{
	color:white;
}