#!/usr/bin/env python
"""
_GetJobSlotsByCMSName_

Oracle implementation of Locations.GetJobSlotsByCMSName
"""

from WMCore.WMBS.MySQL.Locations.GetJobSlotsByCMSName \
    import GetJobSlotsByCMSName as MySQLGetJobSlotsByCMSName

class GetJobSlotsByCMSName(MySQLGetJobSlotsByCMSName):
    pass
