"""
__ListForJobUpdater__

Oracle implementation of ListForJobUpdater

Created on Apr 16, 2013

@author: dballest
"""

from WMCore.WMBS.MySQL.Workflow.ListForJobUpdater import ListForJobUpdater as MySQLListForJobUpdater

class ListForJobUpdater(MySQLListForJobUpdater):
    pass
