#!/usr/bin/env python
"""
_GetOutputMap_

Oracle implementation of Jobs.GetOutputMap
"""

from WMCore.WMBS.MySQL.Jobs.GetOutputMap import GetOutputMap as MySQLGetOutputMap

class GetOutputMap(MySQLGetOutputMap):
    pass
